#!/usr/bin/python

import pgturant.system
import pgturant.constants as constants
import pgturant.log as logger
import os

class App:

   def __init__(self):

       self.initialize()
       return

   def initialize(self):

       self.info = pgturant.system.Info()
       self.system = pgturant.system.System()
       self.logger = logger
       self.error  = None

   def isPgVersionSupported(self, version):

       if constants.SUPPORTED_PG_VERSIONS.has_key(version):
          return constants.SUPPORTED_PG_VERSIONS[version]['enabled']
       else:
          return False

   def errorMsg(self):
       if self.error != None:
          return self.error

       return ''

   def isDistSupported(self, dist=None, release=None):

       if (dist == None):
          dist = self.info['dist']
       if (release == None):
          release = self.info['baserelease']

       if constants.SUPPORTED_DIST.has_key(dist) and constants.SUPPORTED_DIST[dist].has_key(release):
          return constants.SUPPORTED_DIST[dist][release]['enabled']
       else:
          return False

   def isPostgresRunning(self):

       return self.system.isProcessRunning(constants.POSTGRES_PROC_NAME)

   def getSupportedPgVersionList(self):

       supportedVers = filter (lambda vers: constants.SUPPORTED_PG_VERSIONS[vers]['enabled'] == True, constants.SUPPORTED_PG_VERSIONS.keys())
       return supportedVers

   def getSupportedDistributions(self):

       supportedDists = filter (lambda dist: constants.SUPPORTED_DIST[dist] == True, constants.SUPPORTED_DIST.keys())
       return supportedDists

   def isValidPgDirectory(self, dirPath):

       pgVersionFile = os.path.join(dirPath, "PG_VERSION")
       if not os.path.isfile(pgVersionFile): return False
       return True

   def getPgVersionFromDirectory(self, dirPath):

       pgVersionFile = os.path.join(dirPath, "PG_VERSION")
       if not os.path.isfile(pgVersionFile): return ''

       return self.system.getFileContent(pgVersionFile)

   def isThisUpgradeSupported(self, srcVersion, dstVersion):

       try:
          return srcVersion in constants.SUPPORTED_PG_VERSIONS[dstVersion]['paths']
       except:
          return False

   def installPostgresqlRPMS(self, pgVersion):

       rpmList = constants.SUPPORTED_PG_VERSIONS[pgVersion]['packages']
       rpmProfilePath = os.path.join(self.info['baserelease'], pgVersion, self.info['arch'])
       return self.system.installRpms(rpmProfilePath, rpmList)

   def runPgUpgrade(self, srcVersion, dstVersion, srcDir, dstDir, linkMode=False):
       #/usr/pgsql-9.3/bin/pg_upgrade -d /var/lib/pgsql/9.2/data/ -D /tmp/9.3/ -b /usr/pgsql-9.2/bin/ -B /usr/pgsql-9.3/bin/
       baseDbDir = self.system.dirName(dstDir)
       linkModeOption = ''
       logger.printLog("debug", "app: Running pg_upgrade to migrate new database system on %s..."%dstDir)
       if linkMode == True:
          linkModeOption = '-k'
          logger.printLog("debug", "app: Running pg_upgrade in link mode!!")
       initDbCmd = "su postgres -c \"cd %s; /usr/pgsql-%s/bin/pg_upgrade -d %s -D %s -b /usr/pgsql-%s/bin/ -B /usr/pgsql-%s/bin/ %s\""%(baseDbDir, dstVersion, srcDir, dstDir, srcVersion, dstVersion, linkModeOption)
       ret = self.system.executeCommandPopen(initDbCmd)

       if ret[0] != 0:
          self.error = "Error while running pg_upgrade - migration mode!!" + ret[2]
          return False

       logger.printLog("debug", "app: Running pg_upgrade to migrate new database system on %s... completed successfully"%dstDir)
       return True

   def runPgUpgradeCheck(self, srcVersion, dstVersion, srcDir, dstDir, linkMode=False):
       #/usr/pgsql-9.3/bin/pg_upgrade -d /var/lib/pgsql/9.2/data/ -D /tmp/9.3/ -b /usr/pgsql-9.2/bin/ -B /usr/pgsql-9.3/bin/ -c
       baseDbDir = self.system.dirName(dstDir)
       linkModeOption = ''
       if linkMode == True:
          linkModeOption = '-k'
       initDbCmd = "su postgres -c \"cd %s; /usr/pgsql-%s/bin/pg_upgrade -d %s -D %s -b /usr/pgsql-%s/bin/ -B /usr/pgsql-%s/bin/ %s -c\""%(baseDbDir, dstVersion, srcDir, dstDir, srcVersion, dstVersion, linkModeOption)
       ret = self.system.executeCommandPopen(initDbCmd)

       if ret[0] != 0:
          self.error = ret[2]
          self.error = "Error while running pg_upgrade - check mode!!" + ret[2]
          return False

       logger.printLog("debug", "app: Running consistency check command ... completed successfully!!")
       return True

   def initialiseDataDir(self, pgVersion, dbDir):
       #/usr/pgsql-9.3/bin/initdb --pgdata='/tmp/9.3' --auth='ident' --locale='en_US.UTF-8'
       pgVersionFile = os.path.join(dbDir, 'PG_VERSION')
       blessFile     = os.path.join(dbDir, constants.PGTURANT_BLESS_FILE)
       if (os.path.exists(pgVersionFile) and not os.path.exists(blessFile)):
          self.error =  "The pg data directory [%s] already has a database system. Cannot initialise again!!"%dbDir
          return False

       if (os.path.exists(blessFile)):
          logger.printLog("debug", "app: %s database system in %s is already initialised!!"%(pgVersion, dbDir))
          return True

       logger.printLog("debug", "app: Initialising %s database system in %s"%(pgVersion, dbDir))
       baseDbDir = self.system.dirName(dbDir)
       initDbCmd = "su postgres -c \"cd %s; /usr/pgsql-%s/bin/initdb --pgdata='%s' --auth='ident' --locale='en_US.UTF-8'\""%(baseDbDir, pgVersion, dbDir)
       ret = self.system.executeCommand(initDbCmd)

       if ret[0] != 0:
          self.error = ret[1]
          return False

       logger.printLog("debug", "app: Initialising %s database system in %s is successful!!"%(pgVersion, dbDir))
       return True

   def blessDataDir(self, dbDir):

       blessFile = os.path.join(dbDir, constants.PGTURANT_BLESS_FILE)
       logger.printLog("debug", "app: Creating bless file [%s]"%(blessFile))
       blessCmd = "su postgres -c \"touch %s\""%(blessFile)
       ret = self.system.executeCommand(blessCmd)

       if ret[0] != 0:
          self.error = ret[2]
          return False

       logger.printLog("debug", "app: Creating bless file [%s] is successful!!"%(blessFile))
       return True

   def accomplishDataDir(self, dbDir):

       blessFile = os.path.join(dbDir, constants.PGTURANT_BLESS_FILE)
       accomplishFile = os.path.join(dbDir, constants.PGTURANT_ACM_FILE)
       logger.printLog("debug", "app: Creating accomplished file [%s]"%(accomplishFile))
       if os.path.exists(blessFile):
          acmCmd = "su postgres -c \"mv -f %s %s\""%(blessFile, accomplishFile)
       else:
          acmCmd = "su postgres -c \"touch %s\""%(acmCmd)
       ret = self.system.executeCommand(acmCmd)

       if ret[0] != 0:
          self.error = ret[2]
          return False

       logger.printLog("debug", "app: Creating bless file [%s] is successful!!"%(blessFile))
       return True

   def createPosgresqlDataDir(self, dbDir):

       if self.system.createDir(dbDir) == True:
          self.system.fixOwnerships(self.system.dirName(dbDir), 'postgres', 'postgres')
          self.system.fixPermissions(self.system.dirName(dbDir), '700')
